// PCWatchDogTimer.cpp: implementation of the PCWatchDogTimer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PCWatchDOg.h"
#include "CApplicationException.h"

#include "PCWatchDogTimer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
PCWatchDogTimer::PCWatchDogTimer():
m_pCountdownSharedMemory(new CSharedMemoryMapFile),
m_pExternalSerialWatchdog(new CExternalSerialWatchdog),
m_pAdminMemoryMap(new CAdminMemoryMap),
m_pCountdownDlg(new CCountdownDlg),
m_pCountdownLastReading(new CTime)
{	
	// Read Application path of Registry
	string sFilePath = CIniFileReader::GetApplicationPath();
	if (sFilePath.empty())
	{
		// Abnormal condition, something is wrong
		CFileException* pFileEx =
			new CFileException(CFileException::badPath); //All or part of the path is invalid.
		throw (pFileEx);
	};

	// Direct way to get the application title.
	string sFileName = AfxGetAppName();
	string sFileNameExt = sFileName +".ini";

	// Create .INI file class
	m_pIniFileReader = new CIniFileReader(sFilePath+sFileNameExt);

	m_sComPort		= "";
	m_bFailed		= FALSE;

	m_pCountdownDlg->Create(IDD_COUNTDOWN_DIALOG);

	// Initialized in PCWatchDOgDlg
	// m_bEnabled_TS
	// m_bEnabled_TSG
	// m_bEnabled_XBT
	// m_bEnabled_XX
	// m_iCountdownMinutes
	// m_sComPort
	// m_tLastReadingXBT
	// m_tLastReadingTSG
	// m_tLastReadingXX
	// m_tLastReadingTS
	// m_bShutdownHardware
	// m_bShutdownSoftware

	m_pCountdownDlg->CountdownStopped(TRUE);
}

PCWatchDogTimer::~PCWatchDogTimer()
{
	delete m_pIniFileReader;
	m_pIniFileReader = NULL;

	delete m_pCountdownSharedMemory;
	m_pCountdownSharedMemory = NULL;

	delete m_pExternalSerialWatchdog;
	m_pExternalSerialWatchdog = NULL;

	delete m_pAdminMemoryMap;
	m_pAdminMemoryMap = NULL;

	delete m_pCountdownDlg;
	m_pCountdownDlg = NULL;

	delete m_pCountdownLastReading;
	m_pCountdownLastReading = NULL;
}

//////////////////////////////////////////////////////////////////////
// Set/Get
//////////////////////////////////////////////////////////////////////
void PCWatchDogTimer::SetEnabled_XBT(const BOOL& bEnable)
{
	m_bEnabled_XBT = bEnable;
};

BOOL& PCWatchDogTimer::GetEnabled_XBT()
{
	return m_bEnabled_XBT;
};

void PCWatchDogTimer::SetEnabled_TS(const BOOL& bEnable)
{
	m_bEnabled_TS = bEnable;
};

BOOL& PCWatchDogTimer::GetEnabled_TS()
{
	return m_bEnabled_TS;
};

void PCWatchDogTimer::SetEnabled_TSG(const BOOL& bEnable)
{
	m_bEnabled_TSG = bEnable;
};

BOOL& PCWatchDogTimer::GetEnabled_TSG()
{
	return m_bEnabled_TSG;
};

void PCWatchDogTimer::SetEnabled_XX(const BOOL& bEnable)
{
	m_bEnabled_XX = bEnable;
};

BOOL& PCWatchDogTimer::GetEnabled_XX()
{
	return m_bEnabled_XX;
};

void PCWatchDogTimer::SetShutdownHardware(const BOOL& bShutdown)
{
	m_bShutdownHardware = bShutdown;
};

BOOL& PCWatchDogTimer::IsShutdownHardwareChecked()
{
	return m_bShutdownHardware;
};

void PCWatchDogTimer::SetShutdownSoftware(const BOOL& bShutdown)
{
	m_bShutdownSoftware = bShutdown;
};

BOOL& PCWatchDogTimer::IsShutdownSoftwareChecked()
{
	return m_bShutdownSoftware;
};

CSharedMemoryMapFile& PCWatchDogTimer::GetSharedMemoryOfXBT()
{
	return m_SharedMemoryOfXBT;
};

CSharedMemoryMapFile& PCWatchDogTimer::GetSharedMemoryOfTSG()
{
	return m_SharedMemoryOfTSG;
};

CSharedMemoryMapFile& PCWatchDogTimer::GetSharedMemoryOfXX()
{
	return m_SharedMemoryOfXX;
};

CSharedMemoryMapFile& PCWatchDogTimer::GetSharedMemoryOfTS()
{
	return m_SharedMemoryOfTS;
};

void PCWatchDogTimer::SetComPort(const CString& sComPort)
{
	m_sComPort = sComPort;
};

CString& PCWatchDogTimer::GetComPort()
{
	return m_sComPort;
};

void PCWatchDogTimer::SetCountdownMinutes(const int& iCountdownMinutes)
{
	m_iCountdownMinutes = iCountdownMinutes;
};

int& PCWatchDogTimer::GetCountdownMinutes()
{
	return m_iCountdownMinutes;
};

void PCWatchDogTimer::SetLastReadingXBT(const CTime& tLastReading)
{
	m_tLastReadingXBT = tLastReading;
};

CTime& PCWatchDogTimer::GetLastReadingXBT()
{
	return m_tLastReadingXBT;
};
	
void PCWatchDogTimer::SetLastReadingTSG(const CTime& tLastReading)
{
	m_tLastReadingTSG = tLastReading;
};

CTime& PCWatchDogTimer::GetLastReadingTSG()
{
	return m_tLastReadingTSG;
};
	
void PCWatchDogTimer::SetLastReadingXX(const CTime& tLastReading)
{
	m_tLastReadingXX = tLastReading;
};

CTime& PCWatchDogTimer::GetLastReadingXX()
{
	return m_tLastReadingXX;
};
	
void PCWatchDogTimer::SetLastReadingTS(const CTime& tLastReading)
{
	m_tLastReadingTS = tLastReading;
};

CTime& PCWatchDogTimer::GetLastReadingTS()
{
	return m_tLastReadingTS;
};
	
void PCWatchDogTimer::SetFailed(const BOOL& bFailed)
{
	m_bFailed = bFailed;
};

BOOL& PCWatchDogTimer::IsFailed()
{
	return m_bFailed;
};

void PCWatchDogTimer::SetCountdownDlg(CCountdownDlg* pCountdownDlg)
{
	m_pCountdownDlg = pCountdownDlg;
};

CCountdownDlg* PCWatchDogTimer::GetCountdownDlg()
{
	return m_pCountdownDlg;
};

//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////
void PCWatchDogTimer::CheckSharedMemorys()
{
	if (IsFailed() == FALSE)
	{
		// It only reads/writes locations that the user has checked.
		if(m_bEnabled_XBT)
			CheckSharedMemory(m_SharedMemoryOfXBT, m_tLastReadingXBT);

		if(m_bEnabled_TSG)
			CheckSharedMemory(m_SharedMemoryOfTSG, m_tLastReadingTSG);
		
		if(m_bEnabled_XX)
			CheckSharedMemory(m_SharedMemoryOfXX, m_tLastReadingXX);
		
		if(m_bEnabled_TS)
			CheckSharedMemory(m_SharedMemoryOfTS, m_tLastReadingTS);
	}
};

void PCWatchDogTimer::CheckSharedMemory(CSharedMemoryMapFile& pSharedMemory, CTime& tLastReading)
{
	// If it finds "TRUE" in the check list programs shared memory locations,
	// it write "FALSE". Every thing is as it should be.
	// If it finds "FALSE" then it starts a monitoring process.
	CString sSharedMemoryData = "";

	ReadSharedMemory(pSharedMemory, sSharedMemoryData);

	if (sSharedMemoryData == "TRUE")
	{
		ResetSharedMemory(pSharedMemory);
		tLastReading = CTime::GetCurrentTime();

		// Send a command NO OPERATION to External Serial PC Watchdog
		m_pExternalSerialWatchdog->SetCommand(COMAND_0x34);
		m_pExternalSerialWatchdog->SendCommand();

		// Receive response
		CString sResponse = m_pExternalSerialWatchdog->ReadResponse();
	}
	else
	{
		// If it continues to find "FALSE" more than the adjusted period
		CTimeSpan auxTimeSpan = CTime::GetCurrentTime() - tLastReading;

		int iFailedTime = m_pIniFileReader->GetValueI( "Configuration", "FailedTime", 5 );
		if (auxTimeSpan.GetMinutes() >= iFailedTime)
		{
			// Program would be considered failed, and
			// a reboot countdown would start
			m_tBegingCountdown = CTime::GetCurrentTime();

			m_pCountdownSharedMemory = &pSharedMemory;
			m_pCountdownLastReading = &tLastReading;

			CTime tAuxTime(2009, 9, 15, 0, m_iCountdownMinutes, 0);
			m_pCountdownDlg->SetCountdownCounter(tAuxTime);

			m_pCountdownDlg->CountdownStopped(FALSE);
			m_pCountdownDlg->SetTimerEvent();

			SetFailed(TRUE);

			// Show the countdown time in a countdowndialog
			m_pCountdownDlg->ShowWindow(SW_NORMAL);

			RunCountdown();
		};
	};
};

void PCWatchDogTimer::RunCountdown()
{
	MessageBeep(MB_ICONASTERISK);

	CTimeSpan countdownTimeSpan = CTime::GetCurrentTime() - m_tBegingCountdown;

	if (countdownTimeSpan.GetMinutes() < m_iCountdownMinutes)
	{
		// If the "FALSE" is changed to "TRUE" anytime during the countdown interval
		// or the user stop the countdown process, it will be stopped.
		CString sSharedMemoryData;
		ReadSharedMemory(*m_pCountdownSharedMemory, sSharedMemoryData);
		if ((sSharedMemoryData == "TRUE") || (m_pCountdownDlg->IsCountdownStopped() == TRUE))
		{
			SetFailed(FALSE);
			*m_pCountdownLastReading = CTime::GetCurrentTime();
		}
	}
	else
	{
		// Countdown finish
		SetFailed(FALSE);
		*m_pCountdownLastReading = CTime::GetCurrentTime();
		m_pCountdownDlg->OnCancel();

		// Software shutdown is selected
		if (IsShutdownSoftwareChecked() == TRUE)
		{
			// Shut down the system and then restarts the system. . 
			ShutDownWorkstation(EWX_REBOOT);
		}
		// Hardware shutdown is selected
		if (IsShutdownHardwareChecked() == TRUE)
		{
			// Send a command RESET PC NOW to External Serial PC Watchdog
			m_pExternalSerialWatchdog->SetCommand(COMAND_0x20);
			m_pExternalSerialWatchdog->SendCommand();

			// Receive response
			CString sResponse = m_pExternalSerialWatchdog->ReadResponse();
		};
	}
};

void PCWatchDogTimer::ResetSharedMemorys()
{
	// It only reads/writes locations that the user has checked.
	if(m_bEnabled_TS)
		m_pAdminMemoryMap->WritePCWatchDogMemory(m_SharedMemoryOfTS, "FALSE");
	if(m_bEnabled_TSG)
		m_pAdminMemoryMap->WritePCWatchDogMemory(m_SharedMemoryOfTSG, "FALSE");
	if(m_bEnabled_XBT)
		m_pAdminMemoryMap->WritePCWatchDogMemory(m_SharedMemoryOfXBT, "FALSE");
	if(m_bEnabled_XX)
		m_pAdminMemoryMap->WritePCWatchDogMemory(m_SharedMemoryOfXX, "FALSE");
}

void PCWatchDogTimer::ResetSharedMemory(CSharedMemoryMapFile& pSharedMemory)
{
	m_pAdminMemoryMap->WritePCWatchDogMemory(pSharedMemory, "FALSE");
};

void PCWatchDogTimer::WriteSharedMemory(	CSharedMemoryMapFile& pSharedMemory,	/*in*/
											const CString sValue)					/*in*/
{
	m_pAdminMemoryMap->WritePCWatchDogMemory(pSharedMemory, sValue);
};

void PCWatchDogTimer::ReadSharedMemorys(CString& sSharedMemoryDataOfXBT,	/* out*/
										CString& sSharedMemoryDataOfTSG,	/* out*/
										CString& sSharedMemoryDataOfXX,		/* out*/
										CString& sSharedMemoryDataOfTS)		/* out*/
{
	// It only reads/writes locations that the user has checked.
	if(m_bEnabled_TS)
		m_pAdminMemoryMap->ReadPCWatchDogMemory(m_SharedMemoryOfTS, sSharedMemoryDataOfTS);
	if(m_bEnabled_TSG)
		m_pAdminMemoryMap->ReadPCWatchDogMemory(m_SharedMemoryOfTSG, sSharedMemoryDataOfTSG);
	if(m_bEnabled_XBT)
		m_pAdminMemoryMap->ReadPCWatchDogMemory(m_SharedMemoryOfXBT, sSharedMemoryDataOfXBT);
	if(m_bEnabled_XX)
		m_pAdminMemoryMap->ReadPCWatchDogMemory(m_SharedMemoryOfXX, sSharedMemoryDataOfXX);
};

void PCWatchDogTimer::ReadSharedMemory(	CSharedMemoryMapFile& pSharedMemory,	/* in*/
										CString& sSharedMemoryData)				/* out*/
{
	m_pAdminMemoryMap->ReadPCWatchDogMemory(pSharedMemory, sSharedMemoryData);
};

void PCWatchDogTimer::Initialize()
{
	try
	{
		InitSerialConnection();
		InitMemMaps();
	}
	catch (CSerialException* ex)
	{
		// Report the exception error
		AfxMessageBox(ex->GetErrorMessage());
		ex->Delete();
	}
};	

void PCWatchDogTimer::InitSerialConnection()
{
	CString sComPort = GetComPort();
	int iComPort = atoi(sComPort);

	int iMinComPort = m_pIniFileReader->GetValueI( "Configuration", "MinComPort", 0 );
	int iMaxComPort = m_pIniFileReader->GetValueI( "Configuration", "MaxComPort", 30 );
	if( (iComPort >= iMinComPort) && (iComPort <= iMaxComPort) )
	{	
		// The Watchdog requires that the COM port on the PC be set to:
		// 1200 Baund, 8 Data Bits, No Parity Bit, and 2 Stop Bits
		DWORD dwBaudRate	= m_pIniFileReader->GetValueI( "ComPortSetup", "BaudRate", 1200 );
		BYTE DataBits		= m_pIniFileReader->GetValueI( "ComPortSetup", "DataBits", 8 );
		int iInQueue		= m_pIniFileReader->GetValueI( "ComPortSetup", "InQueue", 10000 );
		int iOutQueue		= m_pIniFileReader->GetValueI( "ComPortSetup", "OutQueue", 10000 );

		m_pExternalSerialWatchdog->InitializeSerialPort(iComPort,
														dwBaudRate,
														CSerialPort::NoParity,
														DataBits,
														CSerialPort::TwoStopBits,
														CSerialPort::XonXoffFlowControl,
														iInQueue,
														iOutQueue);
	}
}

void PCWatchDogTimer::InitMemMaps()
{
	string strSharedMemoryName;

	strSharedMemoryName = m_pIniFileReader->GetValueS( "SharedMemoryName", "XBT", "" );
	m_SharedMemoryOfXBT.SetSharedMemoryName(strSharedMemoryName.c_str());

	strSharedMemoryName = m_pIniFileReader->GetValueS( "SharedMemoryName", "TSG", "" );
	m_SharedMemoryOfTSG.SetSharedMemoryName(strSharedMemoryName.c_str());

	strSharedMemoryName = m_pIniFileReader->GetValueS( "SharedMemoryName", "XverXface", "" );
	m_SharedMemoryOfXX.SetSharedMemoryName(strSharedMemoryName.c_str());

	strSharedMemoryName = m_pIniFileReader->GetValueS( "SharedMemoryName", "TimeSrv", "" );
	m_SharedMemoryOfTS.SetSharedMemoryName(strSharedMemoryName.c_str());

	m_pAdminMemoryMap->CreatePCWatchDogMemory(m_SharedMemoryOfXBT);
	m_pAdminMemoryMap->CreatePCWatchDogMemory(m_SharedMemoryOfTSG);
	m_pAdminMemoryMap->CreatePCWatchDogMemory(m_SharedMemoryOfXX);
	m_pAdminMemoryMap->CreatePCWatchDogMemory(m_SharedMemoryOfTS);
}

// The ExitWindowEx API works directly in Win9x/ME,
// but in Windows multi-user systems,
// it requires a special privilege before execution.
void PCWatchDogTimer::ShutDownWorkstation(UINT uFlags)
{
	// 1st enable that privilege for Windows multi-user platforms

	HANDLE hToken;			// handle to process token
	TOKEN_PRIVILEGES tkp;	// pointer to token structure

	// Open the access token associated with the current process
	if (!OpenProcessToken(	GetCurrentProcess(), 
							TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY,
							&hToken))
	{
		int TODO_no_hard_text;
		// Abnormal condition, something is wrong.
		CApplicationException* pApplicationEx = new CApplicationException("OpenProcessToken");
		throw (pApplicationEx);
	};
 
	// Get the LUID for the shutdown privilege. 
	LookupPrivilegeValue(	NULL,
							SE_SHUTDOWN_NAME, 
							&tkp.Privileges[0].Luid); 
 
	tkp.PrivilegeCount = 1;  // one privilege to set    
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
 
	// Get the shutdown privilege for this process. 
 	AdjustTokenPrivileges(	hToken,
							FALSE,
							&tkp,
							0, 
							(PTOKEN_PRIVILEGES)NULL,
							0); 
 
	// Cannot test the return value of AdjustTokenPrivileges. 
 	if (GetLastError() != ERROR_SUCCESS)
	{
		int TODO_no_hard_text;
		// Abnormal condition, something is wrong.
		CApplicationException* pApplicationEx = new CApplicationException("AdjustTokenPrivileges enable failed.");
		throw (pApplicationEx);
	} 

	// Shut down the workstation
	ExitWindowsEx(uFlags, NULL);
}
