// PCWatchDogTimer.h: interface for the PCWatchDogTimer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PCWATCHDOGTIMER_H__820DDAD7_357D_4F2C_8C3B_851CCB3F90BB__INCLUDED_)
#define AFX_PCWATCHDOGTIMER_H__820DDAD7_357D_4F2C_8C3B_851CCB3F90BB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CCountdownDlg.h"	// Added by ClassView
#include "CIniFileReader.h"
#include "CExternalSerialWatchdog.h"
#include "AdminMemoryMap.h"

class PCWatchDogTimer  
{
public:
	PCWatchDogTimer();
	virtual ~PCWatchDogTimer();

	void SetEnabled_XBT(const BOOL& bEnable);
	BOOL& GetEnabled_XBT();

	void SetEnabled_TS(const BOOL& bEnable);
	BOOL& GetEnabled_TS();

	void SetEnabled_TSG(const BOOL& bEnable);
	BOOL& GetEnabled_TSG();

	void SetEnabled_XX(const BOOL& bEnable);
	BOOL& GetEnabled_XX();

	void SetShutdownHardware(const BOOL& bShutdown);
	BOOL& IsShutdownHardwareChecked();

	void SetShutdownSoftware(const BOOL& bShutdown);
	BOOL& IsShutdownSoftwareChecked();

	CSharedMemoryMapFile& GetSharedMemoryOfXBT();
	CSharedMemoryMapFile& GetSharedMemoryOfTSG();
	CSharedMemoryMapFile& GetSharedMemoryOfXX();
	CSharedMemoryMapFile& GetSharedMemoryOfTS();

	void SetComPort(const CString& sComPort);
	CString& GetComPort();

	void SetCountdownMinutes(const int& iCountdownMinutes);
	int& GetCountdownMinutes();

	void SetLastReadingXBT(const CTime& tLastReading);
	CTime& GetLastReadingXBT();

	void SetLastReadingTSG(const CTime& tLastReading);
	CTime& GetLastReadingTSG();

	void SetLastReadingXX(const CTime& tLastReading);
	CTime& GetLastReadingXX();

	void SetLastReadingTS(const CTime& tLastReading);
	CTime& GetLastReadingTS();

	void SetFailed(const BOOL& bFailed);
	BOOL& IsFailed();

	void SetCountdownDlg(CCountdownDlg* pCountdownDlg);
	CCountdownDlg* GetCountdownDlg();


	void CheckSharedMemorys();
	void CheckSharedMemory(	CSharedMemoryMapFile& pSharedMemory,	/* in*/
							CTime& tLastReading);					/* in*/

	void ResetSharedMemorys();
	void ResetSharedMemory(CSharedMemoryMapFile& pSharedMemory);	/* in*/

	void WriteSharedMemory(	CSharedMemoryMapFile& pSharedMemory,	/* in*/
							const CString sValue);					/* in*/

	void ReadSharedMemorys(	CString& sSharedMemoryDataOfXBT,		/* out*/
							CString& sSharedMemoryDataOfTSG,		/* out*/
							CString& sSharedMemoryDataOfXX,			/* out*/
							CString& sSharedMemoryDataOfTS);		/* out*/

	void ReadSharedMemory(	CSharedMemoryMapFile& pSharedMemory,	/* in*/
							CString& sSharedMemoryData);			/* out*/

	void RunCountdown();
	
	void Initialize();
	void InitMemMaps();
	void InitSerialConnection(); // Throw exception in case of an error

private:
	// Intended to either logs off the current user,
	// shuts down the system, or shuts down and restarts the system
	void ShutDownWorkstation(UINT uFlags);

	BOOL						m_bEnabled_TS;
	BOOL						m_bEnabled_TSG;
	BOOL						m_bEnabled_XBT;
	BOOL						m_bEnabled_XX;

	BOOL						m_bShutdownHardware;
	BOOL						m_bShutdownSoftware;

	CSharedMemoryMapFile		m_SharedMemoryOfXBT;
	CSharedMemoryMapFile		m_SharedMemoryOfTSG;
	CSharedMemoryMapFile		m_SharedMemoryOfXX;
	CSharedMemoryMapFile		m_SharedMemoryOfTS;

	CAdminMemoryMap*			m_pAdminMemoryMap;
	CSharedMemoryMapFile*		m_pCountdownSharedMemory;
	CIniFileReader*				m_pIniFileReader;
	CExternalSerialWatchdog*	m_pExternalSerialWatchdog;
	CCountdownDlg*				m_pCountdownDlg; // countdowndialog
	CTime*						m_pCountdownLastReading;

	CString						m_sComPort;
	CTime						m_tLastReadingXBT;
	CTime						m_tLastReadingTSG;
	CTime						m_tLastReadingXX;
	CTime						m_tLastReadingTS;
	BOOL						m_bFailed;
	CTime						m_tBegingCountdown;
	int							m_iCountdownMinutes;
};

#endif // !defined(AFX_PCWATCHDOGTIMER_H__820DDAD7_357D_4F2C_8C3B_851CCB3F90BB__INCLUDED_)
